﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public class KeijzerFunctionFifteen : RegressionToyBenchmark {

    public KeijzerFunctionFifteen() {
      Name = "Keijzer 15 f(x) = 8 / (2 + x^2 + y^2)";
      Description = "Paper: Improving Symbolic Regression with Interval Arithmetic and Linear Scaling" + Environment.NewLine
        + "Authors: Maarten Keijzer" + Environment.NewLine
        + "Function: f(x, y) = 8 / (2 + x^2 + y^2)" + Environment.NewLine
        + "range(train): 20 Testcases x,y = rnd(-3, 3)" + Environment.NewLine
        + "range(test): x,y = [-3:0.01:3]" + Environment.NewLine
        + "Function Set: x + y, x * y, 1/x, -x, sqrt(x)";
      targetVariable = "F";
      inputVariables = new List<string>() { "X", "Y" };
      trainingPartition = new IntRange(0, 20);
      testPartition = new IntRange(21, 621);
    }

    protected override List<double> GenerateTarget(List<List<double>> data) {
      double x, y;
      List<double> results = new List<double>();
      for (int i = 0; i < data[0].Count; i++) {
        x = data[0][i];
        y = data[1][i];
        results.Add(8 / (2 + Math.Pow(x, 2) + Math.Pow(y, 2)));
      }
      return results;
    }

    protected override List<List<double>> GenerateInput() {
      List<List<double>> dataList = new List<List<double>>();
      DoubleRange range = new DoubleRange(-3, 3);

      List<double> oneVariableTestData = RegressionBenchmark.GenerateSteps(range, 0.01);
      List<List<double>> testData = new List<List<double>>() { oneVariableTestData, oneVariableTestData };
      testData = RegressionBenchmark.GenerateAllCombinationsOfValuesInLists(testData);

      for (int i = 0; i < InputVariable.Count; i++) {
        dataList.Add(RegressionBenchmark.GenerateUniformDistributedValues(20, range));
        dataList[i].AddRange(testData[i]);
      }

      return dataList;
    }
  }
}
