﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public class KeijzerFunctionSeven : RegressionToyBenchmark {

    public KeijzerFunctionSeven() {
      Name = "Keijzer 7 f(x) = Sum(1 / i) From 1 to X";
      Description = "Paper: Improving Symbolic Regression with Interval Arithmetic and Linear Scaling" + Environment.NewLine
        + "Authors: Maarten Keijzer" + Environment.NewLine
        + "Function: f(x) = (30 * x * y) / ((x - 10)  * y^2)" + Environment.NewLine
        + "range(train): x = [1:1:50]" + Environment.NewLine
        + "range(test): x = [1:1:120]" + Environment.NewLine
        + "Function Set: x + y, x * y, 1/x, -x, sqrt(x)";
      targetVariable = "Y";
      inputVariables = new List<string>() { "X" };
      trainingPartition = new IntRange(0, 50);
      testPartition = new IntRange(51, 170);
    }

    protected override List<double> GenerateTarget(List<List<double>> data) {
      double x;
      List<double> results = new List<double>();
      for (int i = 0; i < data[0].Count; i++) {
        x = data[0][i];
        results.Add(Enumerable.Range(1, (int)x).Sum(j => 1.0 / j));
      }
      return results;
    }

    protected override List<List<double>> GenerateInput() {
      List<List<double>> dataList = new List<List<double>>();
      dataList.Add(RegressionBenchmark.GenerateSteps(new DoubleRange(1, 50), 1));
      dataList[0].AddRange(RegressionBenchmark.GenerateSteps(new DoubleRange(1, 120), 1));

      return dataList;
    }
  }
}
